/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.readGraph.forObjectSet;

import org.junit.Test;
import org.simantics.db.AsyncReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.AsyncSetListener;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

public class ForObjectSetTest1 extends ExistingDatabaseTest {
	
	private Throwable exception; 
	private Resource relation;
	private int adds = 0;
	private int removes = 0;
	
	@Test
	public void testObjectSet() throws Exception {
		
		Session session = getSession();

		session.syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph graph) throws DatabaseException {
				
				Layer0 b = Layer0.getInstance(graph);
				
				relation = graph.newResource();
				graph.claim(relation, b.SubrelationOf, b.IsRelatedTo);
				graph.claim(relation, b.InverseOf, relation);
				
		        session.syncRequest(new TestAsyncReadRequest() {

		            @Override
		            public void run(AsyncReadGraph graph) throws Throwable {

						graph.forObjectSet(graph.getRootLibrary(), relation, new AsyncSetListener<Resource>() {

							@Override
							public void add(AsyncReadGraph graph, Resource result) {
								synchronized(this) {
									adds++;
								}
							}

							@Override
							public void remove(AsyncReadGraph graph, Resource result) {
								synchronized(this) {
									removes++;
								}
							}

							@Override
							public void exception(AsyncReadGraph graph, Throwable t) {
								exception = t;
							}

							@Override
							public boolean isDisposed() {
								return false;
							}
							
						});
		            	
		            }
		            
		        });
		        
			}
			
		});
		
		session.syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph g) throws DatabaseException {
				
				Layer0 b = Layer0.getInstance(g);
				// Add
				g.claim(g.getRootLibrary(), relation, b.Abstract);
				
			}
			
		});
		
		assert(adds == 1 && removes == 0);

		session.syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph g) throws DatabaseException {
				
				Layer0 b = Layer0.getInstance(g);
				// Nop
				g.claim(g.getRootLibrary(), relation, b.Abstract);
				
			}
			
		});

		assert(adds == 1 && removes == 0);

		session.syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph g) throws DatabaseException {
				
				Layer0 b = Layer0.getInstance(g);
				// Remove
				g.deny(g.getRootLibrary(), relation, b.Abstract);
				
			}
			
		});

		assert(adds == 1 && removes == 1);

		session.syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph g) throws DatabaseException {
				
				Layer0 b = Layer0.getInstance(g);
				// Nop
				g.deny(g.getRootLibrary(), relation, b.Abstract);
				
			}
			
		});

		assert(adds == 1 && removes == 1);

		session.syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph g) throws DatabaseException {
				
				Layer0 b = Layer0.getInstance(g);
				// nop
				g.claim(g.getRootLibrary(), relation, b.Abstract);
				g.deny(g.getRootLibrary(), relation, b.Abstract);
				
			}
			
		});

		assert(adds == 1 && removes == 1);

		if(exception != null) {
			fail("Write transaction threw and exception (" + exception.getMessage() + ") which was not passed through ");
		}
		
	}

}
