package org.simantics.db.tests.api.request;

import org.junit.BeforeClass;
import org.junit.runner.RunWith;
import org.junit.runners.Suite.SuiteClasses;
import org.simantics.db.common.utils.Logger;
import org.simantics.db.testing.common.RegressionSuiteRunner;
import org.simantics.db.testing.common.Tests;
import org.simantics.db.tests.api.request.exception.AsyncProcedureExecuteThrows;
import org.simantics.db.tests.api.request.exception.ExceptionInEqualsTest;
import org.simantics.db.tests.api.request.exception.ExceptionInHashCodeTest;
import org.simantics.db.tests.api.request.exception.GraphSyncAsyncReadPassesDatabaseException;
import org.simantics.db.tests.api.request.exception.GraphSyncAsyncReadThrowsRuntimeException;
import org.simantics.db.tests.api.request.exception.GraphSyncReadThrowsDatabaseException;
import org.simantics.db.tests.api.request.exception.GraphSyncReadThrowsNPE;
import org.simantics.db.tests.api.request.exception.GraphSyncReadThrowsNoSingleResultException;
import org.simantics.db.tests.api.request.exception.GraphSyncReadWithProcedureThrowsDatabaseException;
import org.simantics.db.tests.api.request.exception.GraphSyncReadWithProcedureThrowsNPE;
import org.simantics.db.tests.api.request.exception.GraphSyncReadWithProcedureThrowsNPEInExecute;
import org.simantics.db.tests.api.request.exception.GraphSyncReadWithProcedureThrowsNoSingleResultException;
import org.simantics.db.tests.api.request.exception.ListenerExceptions;
import org.simantics.db.tests.api.request.exception.RecoveryFromExceptedState;
import org.simantics.db.tests.api.request.exception.SessionSyncAsyncReadPassesDatabaseException;
import org.simantics.db.tests.api.request.exception.SessionSyncAsyncReadThrowsRuntimeException;
import org.simantics.db.tests.api.request.exception.SessionSyncReadThrowsDatabaseException;
import org.simantics.db.tests.api.request.exception.SessionSyncReadWithProcedureThrowsDatabaseException;
import org.simantics.db.tests.api.request.exception.SessionSyncReadWithProcedureThrowsNPEInExecute;
import org.simantics.db.tests.api.request.exception.SessionSyncWriteThrowsDatabaseException;
import org.simantics.db.tests.api.request.exception.SessionSyncWriteThrowsRuntimeException;
import org.simantics.db.tests.api.request.external.CachingExternalReads;
import org.simantics.db.tests.api.request.external.ExternalRequestTest;
import org.simantics.db.tests.api.request.external.ExternalRequestTest2;
import org.simantics.db.tests.api.request.external.ExternalRequestTest3;
import org.simantics.db.tests.api.request.external.ExternalRequestTest4;
import org.simantics.db.tests.api.request.listening.DisposedListenerTest;
import org.simantics.db.tests.api.request.listening.EqualListenerDisposeTest;
import org.simantics.db.tests.api.request.listening.ObjectsListeningTest;
import org.simantics.db.tests.api.request.listening.QueryResultCompareTest1;
import org.simantics.db.tests.api.request.misc.AsyncTransactionTest;
import org.simantics.db.tests.api.request.misc.CachedQueryDependencies;
import org.simantics.db.tests.api.request.misc.ComplexReadWriteQueryTest;
import org.simantics.db.tests.api.request.misc.GraphSupportTest;
import org.simantics.db.tests.api.request.misc.MultiListenerTest;
import org.simantics.db.tests.api.request.misc.NoneListenerTest;
import org.simantics.db.tests.api.request.misc.NoneListenerTest2;
import org.simantics.db.tests.api.request.misc.RequestParentTest;
import org.simantics.db.tests.api.request.misc.RequestParentTest2;
import org.simantics.db.tests.api.request.misc.RequestParentTest3;
import org.simantics.db.tests.api.request.misc.RequestParentTest4;
import org.simantics.db.tests.api.request.misc.RequestParentTest5;
import org.simantics.db.tests.api.request.misc.RequestParentTest6;
import org.simantics.db.tests.api.request.misc.RequestProcessorTest1;
import org.simantics.db.tests.api.request.misc.RequestQueuingTest;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest2;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest3;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest4;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest5;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest6;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest7;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest8;
import org.simantics.db.tests.api.request.misc.SyncAsyncSyncTest9;
import org.simantics.db.tests.api.request.misc.SyncRequestTest;
import org.simantics.db.tests.api.request.misc.SyncRequestTest2;
import org.simantics.db.tests.api.request.misc.SyncRequestTest3;
import org.simantics.db.tests.api.request.thread.ThreadingTest1;
import org.simantics.db.tests.api.request.thread.ThreadingTest2;

@RunWith(RegressionSuiteRunner.class)
@SuiteClasses({
	// exception
    AsyncProcedureExecuteThrows.class,
    ExceptionInEqualsTest.class,
    ExceptionInHashCodeTest.class,
    GraphSyncAsyncReadPassesDatabaseException.class,
    GraphSyncAsyncReadThrowsRuntimeException.class,
    GraphSyncReadThrowsDatabaseException.class,
    GraphSyncReadThrowsNoSingleResultException.class,
    GraphSyncReadThrowsNPE.class,
    GraphSyncReadWithProcedureThrowsDatabaseException.class,
    GraphSyncReadWithProcedureThrowsNoSingleResultException.class,
    GraphSyncReadWithProcedureThrowsNPE.class,
    GraphSyncReadWithProcedureThrowsNPEInExecute.class,
    ListenerExceptions.class,
    RecoveryFromExceptedState.class,
    SessionSyncAsyncReadPassesDatabaseException.class,
    SessionSyncAsyncReadThrowsRuntimeException.class,
    SessionSyncReadThrowsDatabaseException.class,
    SessionSyncReadWithProcedureThrowsDatabaseException.class,
    SessionSyncReadWithProcedureThrowsNPEInExecute.class,
    SessionSyncWriteThrowsDatabaseException.class,
    SessionSyncWriteThrowsRuntimeException.class,
    // external
    ExternalRequestTest.class,
    ExternalRequestTest2.class,
    ExternalRequestTest3.class,
    ExternalRequestTest4.class,
    CachingExternalReads.class,
    // listening
    DisposedListenerTest.class,
    EqualListenerDisposeTest.class,
    ObjectsListeningTest.class,
    QueryResultCompareTest1.class,
    // misc
    AsyncTransactionTest.class,
    CachedQueryDependencies.class,
    ComplexReadWriteQueryTest.class,
    GraphSupportTest.class,
    MultiListenerTest.class,
    NoneListenerTest.class,
    NoneListenerTest2.class,
    RequestParentTest.class,
    RequestParentTest2.class,
    RequestParentTest3.class,
    RequestParentTest4.class,
    RequestParentTest5.class,
    RequestParentTest6.class,
    RequestProcessorTest1.class,
    RequestQueuingTest.class,
    SyncAsyncSyncTest.class,
    SyncAsyncSyncTest2.class,
    SyncAsyncSyncTest3.class,
    SyncAsyncSyncTest4.class,
    SyncAsyncSyncTest5.class,
    SyncAsyncSyncTest6.class,
    SyncAsyncSyncTest7.class,
    SyncAsyncSyncTest8.class,
    SyncAsyncSyncTest9.class,
    SyncRequestTest.class,
    SyncRequestTest2.class,
    SyncRequestTest3.class,
    // thread
    ThreadingTest1.class,
    ThreadingTest2.class
})
public class SuiteRequest {

	@BeforeClass
    public static void setUp() {
		try {
			Tests.freshDatabase();
		} catch (Exception e) {
			Logger.defaultLogError(e);
		}
    }
	
}
