/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.exception;

import org.junit.Test;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.testing.base.ExistingDatabaseTest;

public class SessionSyncWriteThrowsDatabaseException extends ExistingDatabaseTest {
	
	private Throwable exception; 
	
	@Test
	public void test() throws Exception {
		
		Session session = getSession();
		
		try {
		
    		session.syncRequest(new WriteRequest() {
    
    			@Override
    			public void perform(WriteGraph graph) throws DatabaseException {
    			    
    			    DatabaseException e = new DatabaseException("Test");
    			    exception = e;
    			    throw e;
    
    			}
    			
    		});
    		
		} catch(DatabaseException e) {
		    
		    if(e != exception) fail("syncRequest did not throw the original exception");
		    
		    // Check that the session bookkeeping is still intact for another write transaction.
		    session.syncRequest(new WriteRequest() {
		        @Override
		        public void perform(WriteGraph graph) throws DatabaseException {
		        }
		    });
		    
		    return;
		    
		} catch(Throwable t) {
		    
	        fail("syncRequest threw unexpected throwable " + t);

		}
		
		fail("syncRequest did not throw");
		
	}

}
