/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.external;

import java.util.concurrent.Semaphore;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Session;
import org.simantics.db.common.request.SingletonPrimitiveRead;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.Listener;
import org.simantics.db.request.Read;
import org.simantics.db.testing.annotation.Fails;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

public class ExternalRequestTest3 extends ExistingDatabaseTest {
	
	private Error exception;
	
	@Test
	@Fails
	public void test() throws Exception {
		
		Session session = getSession();
		
		final Semaphore s = new Semaphore(-4);
		
		session.syncRequest(new Read<String>() {

            @Override
            public String perform(ReadGraph graph) throws DatabaseException {
                
                class Primitive extends SingletonPrimitiveRead<String> {

                    @Override
                    public void register(ReadGraph graph, Listener<String> procedure) {

                        procedure.execute(" Appendix 1");
                        procedure.execute(" Appendix 2");
                        procedure.execute(" Appendix 3");
                        procedure.execute(" Appendix 4");
                        procedure.execute(" Appendix 5");
                        
                    }
                    
                }
                
                Layer0 b = Layer0.getInstance(graph);
                String base = graph.getRelatedValue(graph.getRootLibrary(), b.HasName);
                String appendix = graph.syncRequest(new Primitive());
                return base + appendix;
                
            }
			
		}, new Listener<String>() {

            @Override
            public void exception(Throwable t) {
            }

            @Override
            public void execute(String result) {
                s.release();
            }

            @Override
            public boolean isDisposed() {
                return false;
            }
		    
		});
		
		s.acquire();
		
		if(exception != null) {
			fail("Write transaction threw and exception (" + exception.getMessage() + ") which was not passed through ");
		}
		
	}

}
