/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.listening;

import java.util.Collection;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.procedure.adapter.ListenerAdapter;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.common.request.WriteResultRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.request.Read;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

public class QueryResultCompareTest1 extends ExistingDatabaseTest {

	@Test
    public void testNullArrayCompare() throws Exception {

    	final Session session = getSession();

		final Resource newResource = getSession().syncRequest(new WriteResultRequest<Resource>() {
			@Override
			public Resource perform(WriteGraph graph) throws DatabaseException {
				return graph.newResource();
			}
		});
        
        session.syncRequest(new Read<Resource[]>() {
            @Override
            public Resource[] perform(ReadGraph graph) throws DatabaseException {
                Layer0 b = Layer0.getInstance(graph);
            	Collection<Resource> coll = graph.getObjects(newResource, b.ConsistsOf);
            	if(coll.isEmpty()) return null;
            	else return coll.toArray(Resource.NONE); 
            }
        }, new ListenerAdapter<Resource[]>() {
            @Override
            public boolean isDisposed() {
                return false;
            }
        });

		getSession().syncRequest(new WriteRequest() {
			@Override
			public void perform(WriteGraph graph) throws DatabaseException {
			    Layer0 b = Layer0.getInstance(graph);
			    graph.claim(newResource, b.ConsistsOf, graph.newResource());
			}
		});
        
    }
}
