/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.misc;

import java.util.ArrayList;
import java.util.Arrays;

import org.junit.Test;
import org.simantics.db.Resource;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.primitiverequest.HasStatement;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.common.request.WriteResultRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.Listener;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

public class CachedQueryDependencies extends ExistingDatabaseTest {
	
	@Test
	public void test() throws Exception {
		
		final ArrayList<String> result = new ArrayList<String>();
		
		final Layer0 L0 = Layer0.getInstance(getSession());

		final Resource root = getSession().syncRequest(new WriteResultRequest<Resource>() {

			@Override
			public Resource perform(WriteGraph graph) throws DatabaseException {
				return graph.newResource();
			}
			
		});
		
		getSession().syncRequest(new HasStatement(root, L0.ConsistsOf));		
		getSession().syncRequest(new HasStatement(root, L0.ConsistsOf), new Listener<Boolean>() {

			@Override
			public void execute(Boolean value) {
				if(value) result.add("true");
				else result.add("false");
			}

			@Override
			public void exception(Throwable t) {
			}

			@Override
			public boolean isDisposed() {
				return false;
			}
			
		});		
		
		getSession().syncRequest(new WriteRequest() {

			@Override
			public void perform(WriteGraph graph) throws DatabaseException {
				graph.claim(root, L0.ConsistsOf, root);
			}
			
		});
		
		if(!Arrays.equals(result.toArray(new String[0]), new String[] { "false", "true" })) fail("incorrect result " + result);

	}

}
