/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.request.misc;

import java.util.ArrayList;
import java.util.concurrent.ConcurrentSkipListSet;

import org.junit.Test;
import org.simantics.db.AsyncReadGraph;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.procedure.SyncSetListener;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;

public class SyncAsyncSyncTest5 extends ExistingDatabaseTest {
	
	@Test
	public void test() throws Exception {
		
		Session session = getSession();
		
		final ConcurrentSkipListSet<String> names = new ConcurrentSkipListSet<String>();
		final ArrayList<Resource> resources = new ArrayList<Resource>();
		final Resource root = SyncAsyncSyncUtils.createNames(session, names, resources);

        session.syncRequest(new TestAsyncReadRequest() {

            @Override
            public void run(AsyncReadGraph graph) {
                
                Layer0 b = Layer0.getInstance(graph);
                graph.forObjectSet(root, b.ConsistsOf, new SyncSetListener<Resource>() {
                    
                    @Override
                    public void add(ReadGraph graph, final Resource resource) throws DatabaseException {

                        graph.syncRequest(new TestAsyncReadRequest() {

                            @Override
                            public void run(AsyncReadGraph graph) throws DatabaseException {
                                Layer0 b = Layer0.getInstance(graph);
                                graph.forObjectSet(resource, b.HasName, new SyncSetListener<Resource>() {
                                    
                                    @Override
                                    public void add(ReadGraph graph, final Resource resource) throws DatabaseException {

                                        names.remove(graph.getPossibleValue(resource));

                                    }

                                    @Override
                                    public void exception(ReadGraph graph, Throwable throwable) throws DatabaseException {
                                    }

                                    @Override
                                    public void remove(ReadGraph graph, Resource result) throws DatabaseException {
                                    }

                                    @Override
                                    public boolean isDisposed() {
                                        return false;
                                    }
                                    
                                    @Override
                                    public String toString() {
                                        return "Inner procedure for " + resource.getResourceId();
                                    }
                                    
                                });

                                
                            }
                            
                            @Override
                            public String toString() {
                                return "Inner request for " + resource.getResourceId();
                            }
                            
                        });
                        
                        
                        
                    }

                    @Override
                    public void remove(ReadGraph graph, Resource result) throws DatabaseException {
                    }

                    @Override
                    public void exception(ReadGraph graph, Throwable t) {
                    }

                    @Override
                    public boolean isDisposed() {
                        return false;
                    }
                    
                    @Override
                    public String toString() {
                        return "Outer procedure for RootLibrary";
                    }
                    
                });
                
            }
            
        });
		
		assert(names.isEmpty());
		
	}

}
