/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.api.support.undoRedoSupport;

import java.util.ListIterator;

import org.junit.Test;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.common.utils.OrderedSetUtils;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;
import org.simantics.utils.DataContainer;

/**
 * Test case description missing!
 */
public class UndoTest03 extends ExistingDatabaseTest {

	@Test
    public void testUndo3() throws Exception {

    	Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        final DataContainer<Resource> subject = new DataContainer<Resource>();
        final int size = 3;
        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    Layer0 b = Layer0.getInstance(g);
                    Resource l = OrderedSetUtils.create(g, b.Type);
                    subject.set(l);
                    for (int i=0; i<size; ++i) {
                        Resource el = g.newResource();
                        g.claim(el, b.InstanceOf, null, b.Type);
                        OrderedSetUtils.add(g, l, el);
                    }
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Resource l = subject.get();
                    ListIterator<Resource> it = OrderedSetUtils.iterator(g, l);
                    int count = 0;
                    while (it.hasNext()) {
                        Resource r = it.next();
                        if (DEBUG)
                            System.out.println("Resource " + r);
                        ++count;
                    }
                    if (size != count)
                        fail("Number of ordered set elements doesn't match!");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }

        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph g) throws DatabaseException {
                    g.markUndoPoint();
                    Resource l = subject.get();
                    ListIterator<Resource> a = OrderedSetUtils.iterator(g, l);
                    Resource el = null;
                    if (!a.hasNext())
                        fail("Number of elements doesn't match!");
                    el = a.next();
                    OrderedSetUtils.remove(g, l, el);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Resource l = subject.get();
                    ListIterator<Resource> it = OrderedSetUtils.iterator(g, l);
                    int count = 0;
                    while (it.hasNext()) {
                        Resource r = it.next();
                        if (DEBUG)
                            System.out.println("Resource " + r);
                        ++count;
                    }
                    if (size != count+1)
                        fail("Number of ordered set elements doesn't match!");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }

        support.undo(session, 1);

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph g) throws DatabaseException {
                    Resource l = subject.get();
                    ListIterator<Resource> it = OrderedSetUtils.iterator(g, l);
                    int count = 0;
                    while (it.hasNext()) {
                        Resource r = it.next();
                        if (DEBUG)
                            System.out.println("Resource " + r);
                        ++count;
                    }
                    if (size != count)
                        fail("Number of ordered set elements doesn't match!");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }

    }

}
