package org.simantics.db.tests.regression.bugs;

/*******************************************************************************
 * Copyright (c) 2014 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.ServerInformation;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.db.testing.common.TestBase;

/**
* Tests that database identifier stays the same during purge.
* The tests requires that it is called at least twice and purge is done between the calls.
*/
public class Issue5432Test1 extends ExistingDatabaseTest {
    private final String URI_NAME = "DatabaseId";
    private final String URI_PATH = TestBase.ROOT_LIBRARY_URI + "/" + URI_NAME;
    @Test
    public void test() throws DatabaseException {
        Session session = getSession();
        session.syncRequest(new Init());
    }
    class Init extends WriteRequest {
        @Override
        public void perform(WriteGraph g) throws DatabaseException {
            Resource rl = g.getResource(TestBase.ROOT_LIBRARY_URI);
            Resource testRoot = g.getPossibleResource(URI_PATH);
            ServerInformation si = g.getService(ServerInformation.class);
            String id = si.getDatabaseId();
            if (null == testRoot) {
                testRoot = g.newResource();
                g.claim(testRoot, L0.InstanceOf, L0.Library);
                g.addLiteral(testRoot, L0.HasName, L0.NameOf, L0.String, URI_NAME, Bindings.STRING);
                g.addLiteral(testRoot, L0.DatabaseManagement_HasVersionedId, null, L0.String, id, Bindings.STRING);
                g.claim(rl, L0.ConsistsOf, testRoot);
            }
            String databaseId = g.getRelatedValue(testRoot, L0.DatabaseManagement_HasVersionedId);
            assertTrue(databaseId.equals(id));
        }
    }
}
