/*******************************************************************************
 * Copyright (c) 2007, 2010 Association for Decentralized Information Management
 * in Industry THTH ry.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     VTT Technical Research Centre of Finland - initial API and implementation
 *******************************************************************************/
package org.simantics.db.tests.regression.bugs;

import org.junit.Test;
import org.simantics.databoard.Bindings;
import org.simantics.db.ReadGraph;
import org.simantics.db.Resource;
import org.simantics.db.Session;
import org.simantics.db.WriteGraph;
import org.simantics.db.common.request.ReadRequest;
import org.simantics.db.common.request.WriteRequest;
import org.simantics.db.exception.DatabaseException;
import org.simantics.db.service.UndoRedoSupport;
import org.simantics.db.testing.annotation.Fails;
import org.simantics.db.testing.base.ExistingDatabaseTest;
import org.simantics.layer0.Layer0;
import org.simantics.utils.DataContainer;

public class TicketTest646 extends ExistingDatabaseTest {
    
    @Test
    @Fails
    public void testTicket646() throws Exception {

        Session session = getSession();
        final UndoRedoSupport support = session.getService(UndoRedoSupport.class);
        final DataContainer<Resource> subject = new DataContainer<Resource>();
        final String value = "value";
        final String newValue = "new value";
        final String newValue2 = "new value 2";
        final String newValue3 = "new value 3";
        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    // Use any non-functional relation
                    Layer0 b = Layer0.getInstance(graph);
                    Resource s = graph.newResource();
                    graph.claim(s, b.InstanceOf, b.Entity);
                    graph.claimValue(s, value);
                    subject.set(s);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }
        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    Resource s = subject.get();
                    if (!graph.hasStatement(s))
                        fail("Failed to verify modifications (stm).");
                    String t = graph.getValue(s, Bindings.STRING);
                    if (!t.equals(value))
                        fail("Failed to verify modifications (value).");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }

        try {
            session.syncRequest(new WriteRequest() {
                @Override
                public void perform(WriteGraph graph) throws DatabaseException {
                    Resource s = subject.get();
                    graph.claimValue(s, newValue);
                    graph.claimValue(s, newValue2);
                    graph.claimValue(s, newValue3);
                }
            });
        } catch (Throwable e) {
            fail("Write transaction threw an unknown exception " + e);
        }

        support.undo(session, 1);

        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    Resource s = subject.get();
                    String t = graph.getPossibleValue(s, Bindings.STRING);
                    if (null != t)
                        fail("Failed to verify modifications (value).");
                }
            });
        } catch (Throwable e) {
            fail("Read transaction threw an unexpected exception " + e);
        }

        support.redo(session, 1);
        
        try {
            session.syncRequest(new ReadRequest() {
                @Override
                public void run(ReadGraph graph) throws DatabaseException {
                    Resource s = subject.get();
                    String t = graph.getPossibleValue(s, Bindings.STRING);
                    if (null == t)
                        fail("Failed to verify modifications (value).");
                    if (!t.equals(newValue3))
                        fail("Failed to verify modifications " + t + " != " + newValue3 + ".");
                }
            });
        } catch (Throwable e) {
            e.printStackTrace();
            fail("Read transaction threw an unexpected exception " + e);
        }

    }
}